@@ -67,7 +67,7 @@ Bot Ready
 - ChatEvent: :const:`~twitchAPI.types.ChatEvent.READY`
 - Payload: :const:`~twitchAPI.chat.EventData`
 
-This Event is triggered when the bot is stared up and ready to join channels.
+This Event is triggered when the bot is started up and ready to join channels.
 
 Message send
 ============
@@ -161,6 +161,14 @@ Triggered when someone whispers to your bot.
 
 .. note:: You need the :const:`~twitchAPI.types.AuthScope.WHISPERS_READ` Auth Scope to receive this Event.
 
+Server notice
+============================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.NOTICE`
+- Payload: :const:`~twitchAPI.chat.NoticeEvent`
+
+Triggered when server sends a notice message.
+
 
 ************
 Code example
@@ -266,12 +274,12 @@ import aiohttp
 from twitchAPI.twitch import Twitch
 from twitchAPI.object import TwitchUser
 from twitchAPI.helper import TWITCH_CHAT_URL, first, RateLimitBucket, RATE_LIMIT_SIZES
-from twitchAPI.types import ChatRoom, TwitchBackendException, AuthType, AuthScope, ChatEvent, MissingScopeException, UnauthorizedException
+from twitchAPI.types import ChatRoom, TwitchBackendException, AuthType, AuthScope, ChatEvent, UnauthorizedException
 
 from typing import List, Optional, Union, Callable, Dict, Awaitable, Any
 
 __all__ = ['Chat', 'ChatUser', 'EventData', 'ChatMessage', 'ChatCommand', 'ChatSub', 'ChatRoom', 'ChatEvent', 'RoomStateChangeEvent',
-           'JoinEvent', 'JoinedEvent', 'LeftEvent', 'ClearChatEvent', 'WhisperEvent', 'MessageDeletedEvent']
+           'JoinEvent', 'JoinedEvent', 'LeftEvent', 'ClearChatEvent', 'WhisperEvent', 'MessageDeletedEvent', 'NoticeEvent']
 
 
 class ChatUser:
@@ -512,6 +520,23 @@ class WhisperEvent(EventData):
         """The user that DMed your bot"""
         return ChatUser(self.chat, self._parsed)
 
+class NoticeEvent(EventData):
+    """Represents a server notice"""
+    
+    def __init__(self, chat, parsed):
+        super(NoticeEvent, self).__init__(chat)
+        self._room_name = parsed['command']['channel'][1:]
+        """The name of the chat room the notice is from"""
+        self.msg_id: str = parsed['tags'].get('msg-id')
+        """Message ID of the notice, `Msg-id reference <https://dev.twitch.tv/docs/irc/msg-id/>`__"""
+        self.message: str = parsed['parameters']
+        """Description for the msg_id"""
+
+    @property
+    def room(self) -> Optional[ChatRoom]:
+        """The room this notice is from"""
+        return self.chat.room_cache.get(self._room_name)
+        
 
 COMMAND_CALLBACK_TYPE = Callable[[ChatCommand], Awaitable[None]]
 EVENT_CALLBACK_TYPE = Callable[[Any], Awaitable[None]]
@@ -540,20 +565,20 @@ class Chat:
         self.connection_url: str = connection_url if connection_url is not None else TWITCH_CHAT_URL
         self.ping_frequency: int = 120
         self.ping_jitter: int = 4
-        self._initial_channel: Optional[List[str]] = initial_channel
         self.listen_confirm_timeout: int = 30
         self.reconnect_delay_steps: List[int] = [0, 1, 2, 4, 8, 16, 32, 64, 128]
         self.log_no_registered_command_handler: bool = True
         """Controls if instances of commands being issued in chat where no handler exists should be logged. |default|:code:`True`"""
         self.__connection = None
         self._session = None
-        self.__socket_thread: threading.Thread = None
+        self.__socket_thread: Optional[threading.Thread] = None
         self.__running: bool = False
         self.__socket_loop = None
         self.__startup_complete: bool = False
         self.__tasks = None
         self._ready = False
         self._send_buckets = {}
+        self._join_target = initial_channel if initial_channel is not None else []
         self._join_bucket = RateLimitBucket(10, 2000 if is_verified_bot else 20, 'channel_join', self.logger)
         self.__waiting_for_pong: bool = False
         self._event_handler = {}
@@ -563,8 +588,9 @@ class Chat:
         self._room_leave_locks = []
         self._closing: bool = False
         self.join_timeout: int = 10
-        self._mod_status_cache = {}
         """Time in seconds till a channel join attempt times out"""
+        self._mod_status_cache = {}
+        self._subscriber_status_cache = {}
 
     def __await__(self):
         t = asyncio.create_task(self._get_username())
@@ -640,7 +666,6 @@ class Chat:
         return parsed_message
 
     def _parse_irc_parameters(self, raw_parameters_component: str, command):
-        idx = 0
         command_parts = raw_parameters_component[len(self._prefix)::].strip()
         try:
             params_idx = command_parts.index(' ')
@@ -651,7 +676,8 @@ class Chat:
         command['bot_command_params'] = command_parts[params_idx:].strip()
         return command
 
-    def _parse_irc_source(self, raw_source_component: str):
+    @staticmethod
+    def _parse_irc_source(raw_source_component: str):
         if raw_source_component is None:
             return None
         source_parts = raw_source_component.split('!')
@@ -660,7 +686,8 @@ class Chat:
             'host': source_parts[1] if len(source_parts) == 2 else source_parts[0]
         }
 
-    def _parse_irc_tags(self, raw_tags_component: str):
+    @staticmethod
+    def _parse_irc_tags(raw_tags_component: str):
         tags_to_ignore = ('client-nonce', 'flags')
         parsed_tags = {}
 
@@ -795,7 +822,10 @@ class Chat:
         self._closing = True
 
     async def __connect(self, is_startup=False):
-        self.logger.debug('connecting...')
+        if is_startup:
+            self.logger.debug('connecting...')
+        else:
+            self.logger.debug('reconnecting...')
         if self.__connection is not None and not self.__connection.closed:
             await self.__connection.close()
         retry = 0
@@ -832,7 +862,8 @@ class Chat:
         # keep loop alive
         self.__socket_loop.run_until_complete(self._keep_loop_alive())
 
-    def _task_callback(self, task: asyncio.Task):
+    @staticmethod
+    def _task_callback(task: asyncio.Task):
         e = task.exception()
         if e is not None:
             traceback.print_exception(type(e), e, e.__traceback__, file=sys.stderr)
@@ -876,18 +907,30 @@ class Chat:
                             asyncio.ensure_future(handler(parsed))
                 elif message.type == aiohttp.WSMsgType.CLOSED:
                     self.logger.debug('websocket is closing')
-                    break
+                    if self.__running:
+                        try:
+                            await self._handle_base_reconnect()
+                        except TwitchBackendException:
+                            self.logger.exception('Connection to chat websocket lost and unable to reestablish connection!')
+                            break
+                    else:
+                        break
                 elif message.type == aiohttp.WSMsgType.ERROR:
-                    self.logger.warning('error in websocket')
+                    self.logger.warning('error in websocket: ' + str(self.__connection.exception()))
                     break
         except CancelledError:
             # we are closing down!
             # print('we are closing down!')
             return
 
+    async def _handle_base_reconnect(self):
+        await self.__connect(is_startup=False)
+        await self.__task_startup()
+
+    # noinspection PyUnusedLocal
     async def _handle_reconnect(self, parsed: dict):
         self.logger.info('got reconnect request...')
-        await self.__connect(is_startup=False)
+        await self._handle_base_reconnect()
         self.logger.info('reconnect completed')
 
     async def _handle_whisper(self, parsed: dict):
@@ -903,8 +946,11 @@ class Chat:
             t.add_done_callback(self._task_callback)
 
     async def _handle_notice(self, parsed: dict):
+        e = NoticeEvent(self, parsed)
+        for handler in self._event_handler.get(ChatEvent.NOTICE, []):
+            t = asyncio.ensure_future(handler(e))
+            t.add_done_callback(self._task_callback)
         self.logger.debug(f'got NOTICE for channel {parsed["command"]["channel"]}: {parsed["tags"]["msg-id"]}')
-        pass
 
     async def _handle_clear_msg(self, parsed: dict):
         ev = MessageDeletedEvent(self, parsed)
@@ -990,22 +1036,26 @@ class Chat:
         if parsed['tags'].get('badges') is not None:
             is_broadcaster = parsed['tags']['badges'].get('broadcaster') is not None
         self._mod_status_cache[parsed['command']['channel'][1:]] = 'mod' if parsed['tags']['mod'] == '1' or is_broadcaster else 'user'
+        self._subscriber_status_cache[parsed['command']['channel'][1:]] = 'sub' if parsed['tags']['subscriber'] == '1' else 'non-sub'
 
     async def _handle_ping(self, parsed: dict):
         self.logger.debug('got PING')
         await self._send_message('PONG ' + parsed['parameters'])
 
+    # noinspection PyUnusedLocal
     async def _handle_ready(self, parsed: dict):
         self.logger.debug('got ready event')
         dat = EventData(self)
+        was_ready = self._ready
         self._ready = True
-        if self._initial_channel is not None and len(self._initial_channel) > 0:
-            _failed = await self.join_room(self._initial_channel)
+        if self._join_target is not None and len(self._join_target) > 0:
+            _failed = await self.join_room(self._join_target)
             if len(_failed) > 0:
                 self.logger.warning(f'failed to join the following channel of the initial following list: {", ".join(_failed)}')
-        for h in self._event_handler.get(ChatEvent.READY, []):
-            t = asyncio.ensure_future(h(dat))
-            t.add_done_callback(self._task_callback)
+        if not was_ready:
+            for h in self._event_handler.get(ChatEvent.READY, []):
+                t = asyncio.ensure_future(h(dat))
+                t.add_done_callback(self._task_callback)
 
     async def _handle_msg(self, parsed: dict):
         self.logger.debug('got new message, call handler')
@@ -1026,7 +1076,7 @@ class Chat:
 
     async def __task_startup(self):
         await self._send_message('CAP REQ :twitch.tv/membership twitch.tv/tags twitch.tv/commands')
-        await self._send_message(f'PASS oauth:{self.twitch.get_user_auth_token()}')
+        await self._send_message(f'PASS oauth:{await self.twitch.get_refreshed_user_auth_token()}')
         await self._send_message(f'NICK {self.username}')
         self.__startup_complete = True
 
@@ -1118,6 +1168,20 @@ class Chat:
         if room[0] == '#':
             room = room[1:]
         return self._mod_status_cache.get(room.lower(), 'user') == 'mod'
+    
+    def is_subscriber(self, room: Union[str, ChatRoom]) -> bool:
+        """Check if chat bot is a subscriber in a channel
+
+        :param room: The chat room you want to check if bot is a subscriber in.
+            This can either be a instance of :const:`~twitchAPI.types.ChatRoom` or a string with the room name (either with leading # or without)
+        :return: Returns True if chat bot is a subscriber """
+        if isinstance(room, ChatRoom):
+            room = room.name
+        if room is None or len(room) == 0:
+            raise ValueError('please specify a room')
+        if room[0] == '#':
+            room = room[1:]
+        return self._subscriber_status_cache.get(room.lower(), 'user') == 'sub'
 
     def is_in_room(self, room: Union[str, ChatRoom]) -> bool:
         """Check if the bot is currently in the given chat room
@@ -1160,6 +1224,7 @@ class Chat:
         while any([r in self._room_join_locks for r in target]) and timeout > datetime.datetime.now():
             await asyncio.sleep(0.01)
         failed_to_join = [r for r in self._room_join_locks if r in target]
+        self._join_target.extend([x for x in target if x not in failed_to_join])
         for r in failed_to_join:
             self._room_join_locks.remove(r)
         return failed_to_join
@@ -1217,6 +1282,9 @@ class Chat:
         for r in target:
             self._room_leave_locks.append(r)
         await self._send_message(f'PART {room_str}')
+        for x in target:
+            if x in self._join_target:
+                self._join_target.remove(x)
         # wait to leave all rooms
         while any([r in self._room_leave_locks for r in target]):
             await asyncio.sleep(0.01)
