@@ -39,13 +39,15 @@ Code Example
     from twitchAPI.twitch import Twitch
     from twitchAPI.helper import first
     from twitchAPI.eventsub import EventSub
+    from twitchAPI.oauth import UserAuthenticator
+    from twitchAPI.types import AuthScope
     import asyncio
 
-
     TARGET_USERNAME = 'target_username_here'
     EVENTSUB_URL = 'https://url.to.your.webhook.com'
     APP_ID = 'your_app_id'
     APP_SECRET = 'your_app_secret'
+    TARGET_SCOPES = [AuthScope.MODERATOR_READ_FOLLOWERS]
 
 
     async def on_follow(data: dict):
@@ -58,6 +60,12 @@ Code Example
         twitch = await Twitch(APP_ID, APP_SECRET)
         user = await first(twitch.get_users(logins=TARGET_USERNAME))
 
+        # the user has to authenticate once using the bot with our intended scope.
+        # since we do not need the resulting token after this authentication, we just discard the result we get from authenticate()
+        # Please read up the UserAuthenticator documentation to get a full view of how this process works
+        auth = UserAuthenticator(twitch, TARGET_SCOPES)
+        await auth.authenticate()
+
         # basic setup, will run on port 8080 and a reverse proxy takes care of the https and certificate
         event_sub = EventSub(EVENTSUB_URL, APP_ID, 8080, twitch)
 
@@ -67,8 +75,9 @@ Code Example
         # start the eventsub client
         event_sub.start()
         # subscribing to the desired eventsub hook for our user
-        # the given function will be called every time this event is triggered
-        await event_sub.listen_channel_follow(user.id, on_follow)
+        # the given function (in this example on_follow) will be called every time this event is triggered
+        # the broadcaster is a moderator in their own channel by default so specifying both as the same works in this example
+        await event_sub.listen_channel_follow_v2(user.id, user.id, on_follow)
 
         # eventsub will run in its own process
         # so lets just wait for user input before shutting it all down again
